"""
Benchmark functions to be used with Airspeed Velocity.
"""
import numpy as np

import locan as lc

rng = np.random.default_rng(seed=1)


class BenchmarkResample:
    """
    Benchmarks for data.metadata_pb2.Metadata objects
    """

    def __init__(self):
        self.locdata = None
        self.new_locdata = None

    def setup(self):
        self.locdata = lc.simulate_uniform(n_samples=1000, seed=rng)
        self.locdata.data["uncertainty"] = 0.01
        self.locdata.data["other_property"] = 111

    def time_resample(self):
        self.new_locdata = lc.resample(self.locdata, n_samples=1000, seed=rng)

    def mem_locdata(self):
        return self.new_locdata


if __name__ == "__main__":
    class_ = BenchmarkResample()
    class_.setup()
    print(class_.collection.data)



"""
Utility script to print locan.__all__ elements.

These items need to be explicitly re-exported in locan.__init__
to satisfy mypy.
"""
from importlib import import_module


def main() -> None:
    submodules: list[str] = [
        "analysis",
        "configuration",
        "constants",
        "data",
        "datasets",
        "dependencies",
        "gui",
        "locan_io",
        "simulation",
        "tests",
        "utils",
        "visualize",
    ]
    for submodule in submodules:
        module = import_module(name=f".{submodule}", package="locan")
        lines = ",\n".join([f"{item} as {item}" for item in sorted(module.__all__)])
        text = (
            f"from {module.__name__} import (  # type:ignore[attr-defined]\n"
            + lines
            + "\n)"
        )
        print(text)


if __name__ == "__main__":
    main()



"""

Visualize localization data.

This module provides functions to visualize localization data
with matplotlib or napari.

Submodules:
-----------

.. autosummary::
   :toctree: ./

   colormap
   render
   render_mpl
   render_napari
   transform

"""
from __future__ import annotations

from locan.visualize import colormap, render, render_mpl, render_napari, transform

from .colormap import *
from .render import *
from .render_mpl import *
from .render_napari import *
from .transform import *

__all__: list[str] = []
__all__.extend(colormap.__all__)
__all__.extend(render.__all__)
__all__.extend(render_mpl.__all__)
__all__.extend(render_napari.__all__)
__all__.extend(transform.__all__)



"""
Localization data.

This module contains classes and functions to deal with single-molecule localization data.
All functions provide or modify LocData objects.

Submodules:
-----------

.. autosummary::
   :toctree: ./

   locdata
   properties
   hulls
   rois
   region
   region_utils
   register
   aggregate
   filter
   transform
   cluster
   tracking
   metadata_utils
   validation

"""
from __future__ import annotations

from importlib import import_module

from locan.data.aggregate import *
from locan.data.cluster import *
from locan.data.filter import *
from locan.data.hulls import *
from locan.data.locdata import *
from locan.data.metadata_utils import *
from locan.data.properties import *
from locan.data.region import *
from locan.data.region_utils import *
from locan.data.register import *
from locan.data.rois import *
from locan.data.tracking import *
from locan.data.transform import *
from locan.data.validation import *

submodules: list[str] = [
    "aggregate",
    "cluster",
    "filter",
    "hulls",
    "locdata",
    "metadata_utils",
    "properties",
    "region",
    "region_utils",
    "register",
    "rois",
    "tracking",
    "transform",
    "validation",
]

__all__: list[str] = []

for submodule in submodules:
    module_ = import_module(name=f".{submodule}", package="locan.data")
    __all__.extend(module_.__all__)



import os

import locan
from locan.__main__ import main


def test_version():
    try:
        assert isinstance(locan._version.version, str)
        assert isinstance(locan._version.version_tuple, tuple)
    except AttributeError:
        pass
    assert locan.__version__
    # print(locan.__version__)
    # print(len(dir(locan)))
    # print(dir(locan))
    # print(len(locan.__all__))
    # print(locan.__all__)

    assert all(item in locan.__dict__ for item in locan.__all__)
    assert all(item in locan.__dict__ for item in locan.submodules)
    # to double check:
    # stoplist = ['annotations', 'logging', 'import_module', 'Path', '_version',
    # 'locdata_id', 'submodules', 'submodule', 'locan_types', 'module_', 'scripts']
    # all([
    #     item in stoplist
    #     for item in locan.__dict__
    #     if (item not in locan.__all__
    #         and item not in locan.submodules
    #         and not item.startswith("__")
    #         )
    # ])


def test_entrypoint(capfd):
    main([])
    captured = capfd.readouterr()
    assert captured.out[:47] == "This is the command line entry point for locan."


def test_entrypoint_from_sys(capfd):
    exit_status = os.system("locan")
    captured = capfd.readouterr()
    assert captured.out[:47] == "This is the command line entry point for locan."
    assert exit_status == 0
